// Copyright (C) 2008 Chong Luo
// Licensed under the GNU LGPL Version 2.1.
//
//
// First added:  2008
// Last changed: 2008-10-17
//

#include <dolfin.h>
#include "Elastomer2d.h"
  
using namespace dolfin;

int mesh_size=16;

// Dirichlet boundary condition for displacement
class DspDirichletBC : public Function
{
public:
  void eval(real* values, const real* x) const
  {
	values[0] = 0; 
	values[1] = 0;
  }
};

// Dirichlet boundary condition for director
class OrtDirichletBC : public Function
{
public:
  void eval(real* values, const real* x) const
  {
	values[0] = 0.0;
	values[1] = 1.0;
  }
};

// Sub domain for Dirichlet boundary condition
class DirichletBoundary : public SubDomain
{
  bool inside(const real* x, bool on_boundary) const
  {
    return std::abs(std::abs(x[0]-0.5)-0.5) < DOLFIN_EPS && on_boundary; 
  }
};

// User defined nonlinear problem 
class MyNonlinearProblem : public NonlinearProblem
{
  public:

    // Constructor 
    MyNonlinearProblem(Mesh& mesh, SubDomain& dirichlet_boundary,
	                   Function& u, DspDirichletBC& uD, OrtDirichletBC& nD) 
                      : reset_Jacobian(true)
    {
      // Create forms
        V = new Elastomer2dFunctionSpace(mesh);

        a = new Elastomer2dBilinearForm(*V, *V);
        Elastomer2dBilinearForm* _a = dynamic_cast<Elastomer2dBilinearForm*>(a);
        if (!_a) error("Problem in downcast of Elastomer2dBilinearForm");

        L = new Elastomer2dLinearForm(*V);
        Elastomer2dLinearForm* _L = dynamic_cast<Elastomer2dLinearForm*>(L);
        if (!_L) error("Problem in downcast of Elastomer2dLinearForm");
        _L->U0 = u;

      // Create boundary conditions
      MeshFunction<unsigned int> sub_domains(mesh, mesh.topology().dim() -1); 
      sub_domains = 2;
      dirichlet_boundary.mark(sub_domains, 5); 
      // Define sub systems for boundary
      SubSpace un(*V, 0, 0);
      SubSpace u12(un, 0);
	  SubSpace n12(un, 1);
      bcDsp = new DirichletBC(u12, uD, sub_domains, 5);
      bcOrt = new DirichletBC(n12, nD, sub_domains, 5);
	  bcDsp->dim();
	  bcOrt->dim();
	}

    void F(GenericVector& b, const GenericVector& x)
    {
    }

    // User defined assemble of Jacobian matrix 
    void J(GenericMatrix& A, const GenericVector& x)
    {
    }


    // Destructor 
    ~MyNonlinearProblem()
    {
	  delete V;
      delete a;
      delete L;
      delete bcDsp;
      delete bcOrt;
    }

//  private:

    // Pointers to FunctionSpace and forms
    FunctionSpace* V;
    Form* a;
    Form* L;

    // Pointers to boundary conditions
    DirichletBC* bcDsp;
    DirichletBC* bcOrt;

    bool reset_Jacobian;
};

int main(int argc, char* argv[])
{
  dolfin_init(argc, argv);
 
  // Create mesh
  UnitSquare mesh(mesh_size, mesh_size);
 
  // Dirichlet boundary conditions
  DirichletBoundary dirichlet_boundary;
  DspDirichletBC uD;
  OrtDirichletBC nD;

  // Solution functions
  Function u; 

  MyNonlinearProblem nonlinear_problem(mesh, dirichlet_boundary, u, uD, nD);
  cout << "check bcDsp after initialization" << endl;
  nonlinear_problem.bcDsp->dim();
  cout << "check bcOrt after initialization" << endl;
  nonlinear_problem.bcOrt->dim();

  return 0;
}
